# -*- coding: utf-8 -*-
from __future__ import unicode_literals

try:
    # Python 3
    from urllib.error import URLError
    str_cls = str
except (ImportError):
    # Python 2
    from urllib2 import URLError
    str_cls = unicode


class UndefinitiveError(ValueError):

    """
    An error representing an undefinitive answer to determining what VAT rate
    a user is subject to
    """

    pass


class InvalidError(ValueError):

    """
    An error representing an invalid VAT ID
    """

    pass


class WebServiceError(URLError):

    """
    If there was an unexpected result from a call to one of the VAT validation
    APIs
    """

    pass


class WebServiceUnavailableError(WebServiceError):

    """
    If the VIES service VAT ID check was unavailable at the time of the request
    """

    pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from decimal import Decimal

try:
    # Python 2
    str_cls = unicode
except (NameError):
    # Python 3
    str_cls = str

from . import rates
from .errors import UndefinitiveError


def calculate_rate(country_code, subdivision, city, address_country_code=None, address_exception=None):
    """
    Calculates the VAT rate from the data returned by a GeoLite2 database

    :param country_code:
        Two-character country code

    :param subdivision:
        The first subdivision name

    :param city:
        The city name

    :param address_country_code:
        The user's country_code, as detected from billing_address or
        declared_residence. This prevents an UndefinitiveError from being
        raised.

    :param address_exception:
        The user's exception name, as detected from billing_address or
        declared_residence. This prevents an UndefinitiveError from being
        raised.

    :raises:
        ValueError - if country code is not two characers, or subdivision or city are not strings
        UndefinitiveError - when no address_country_code and address_exception are provided and the geoip2 information is not specific enough

    :return:
        A tuple of (Decimal percentage rate, country code, exception name [or None])
    """

    if not country_code or not isinstance(country_code, str_cls) or len(country_code) != 2:
        raise ValueError('Invalidly formatted country code')

    if not isinstance(subdivision, str_cls):
        raise ValueError('Subdivision is not a string')

    if not isinstance(city, str_cls):
        raise ValueError('City is not a string')

    country_code = country_code.upper()
    subdivision = subdivision.lower()
    city = city.lower()

    if country_code not in rates.BY_COUNTRY:
        return (Decimal('0.0'), country_code, None)

    country_default = rates.BY_COUNTRY[country_code]['rate']

    if country_code not in GEOIP2_EXCEPTIONS:
        return (country_default, country_code, None)

    exceptions = GEOIP2_EXCEPTIONS[country_code]
    for matcher in exceptions:
        # Subdivision-only match
        if isinstance(matcher, str_cls):
            sub_match = matcher
            city_match = None
        else:
            sub_match, city_match = matcher

        if sub_match != subdivision:
            continue

        if city_match and city_match != city:
            continue

        info = exceptions[matcher]
        exception_name = info['name']
        if not info['definitive']:
            if address_country_code is None:
                raise UndefinitiveError('It is not possible to determine the users VAT rates based on the information provided')

            if address_country_code != country_code:
                continue

            if address_exception != exception_name:
                continue

        rate = rates.BY_COUNTRY[country_code]['exceptions'][exception_name]
        return (rate, country_code, exception_name)

    return (country_default, country_code, None)


# A dictionary that maps information from the GeoLite2 databases to VAT
# exceptions. Top level keys are country codes, each pointing to a dictionary
# with keys that are either a tuple of subdivision name and city name, or just
# a string of subdivision name.
#
# There is a key 'definitive' that indicates is the match is sufficiently
# specific to fully map to the exemption. If 'definitive' is False, other
# methods must be used to obtain place of supply proof.
GEOIP2_EXCEPTIONS = {
    'AT': {
        ('tyrol', 'reutte'): {
            'name': 'Jungholz',
            'definitive': False
        },
        ('vorarlberg', 'mittelberg'): {
            'name': 'Mittelberg',
            'definitive': True
        }
    },
    'DE': {
        ('baden-württemberg region', 'konstanz'): {
            'name': 'Büsingen am Hochrhein',
            'definitive': False
        },
        ('schleswig-holstein', 'pinneberg'): {
            'name': 'Heligoland',
            'definitive': False
        }
    },
    'ES': {
        'canary islands': {
            'name': 'Canary Islands',
            'definitive': True
        },
        'ceuta': {
            'name': 'Ceuta',
            'definitive': True
        },
        'melilla': {
            'name': 'Melilla',
            'definitive': True
        }
    },
    'GR': {
        # There is no direct entry for Mount Athos, so we just flag the
        # Central Macedonia region since it is part of that
        'central macedonia': {
            'name': 'Mount Athos',
            'definitive': False
        }
    },
    'IT': {
        ('lombardy', 'livigno'): {
            'name': 'Livigno',
            'definitive': True
        },
        # There are no entries that cover Campione d'Italia, so instead we
        # just flag the whole region of Lombardy as not definitive.
        'lombardy': {
            'name': "Campione d'Italia",
            'definitive': False
        }
    },
    'PT': {
        'azores': {
            'name': 'Azores',
            'definitive': True
        },
        'madeira': {
            'name': 'Madeira',
            'definitive': True
        }
    }
}



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from xml.etree import ElementTree
import cgi
from decimal import Decimal

try:
    # Python 3
    from urllib.request import urlopen
    str_cls = str
except (ImportError):
    # Python 2
    from urllib2 import urlopen
    str_cls = unicode

try:
    from money import xrates
except (ImportError):
    xrates = None

from .errors import WebServiceError
builtin_format = format


def fetch():
    """
    Fetches the latest exchange rate info from the European Central Bank. These
    rates need to be used for displaying invoices since some countries require
    local currency be quoted. Also useful to store the GBP rate of the VAT
    collected at time of purchase to prevent fluctuations in exchange rates from
    significantly altering the amount of tax due the HMRC (if you are using them
    for VAT MOSS).

    :return:
        A dict with string keys that are currency codes and values that are
        Decimals of the exchange rate with the base (1.0000) being the Euro
        (EUR). The following currencies are included, based on this library
        being build for EU and Norway VAT, plus USD for the author:
         - BGN
         - CZK
         - DKK
         - EUR
         - GBP
         - HUF
         - HRK
         - NOK
         - PLN
         - RON
         - SEK
         - USD
    """

    response = urlopen('https://www.ecb.europa.eu/stats/eurofxref/eurofxref-daily.xml')
    _, params = cgi.parse_header(response.headers['Content-Type'])
    if 'charset' in params:
        encoding = params['charset']
    else:
        encoding = 'utf-8'

    return_xml = response.read().decode(encoding)

    # Example return data
    #
    # <gesmes:Envelope xmlns:gesmes="http://www.gesmes.org/xml/2002-08-01" xmlns="http://www.ecb.int/vocabulary/2002-08-01/eurofxref">
    #     <gesmes:subject>Reference rates</gesmes:subject>
    #     <gesmes:Sender>
    #         <gesmes:name>European Central Bank</gesmes:name>
    #     </gesmes:Sender>
    #     <Cube>
    #         <Cube time="2015-01-09">
    #             <Cube currency="USD" rate="1.1813"/>
    #             <Cube currency="JPY" rate="140.81"/>
    #             <Cube currency="BGN" rate="1.9558"/>
    #             <Cube currency="CZK" rate="28.062"/>
    #             <Cube currency="DKK" rate="7.4393"/>
    #             <Cube currency="GBP" rate="0.77990"/>
    #             <Cube currency="HUF" rate="317.39"/>
    #             <Cube currency="PLN" rate="4.2699"/>
    #             <Cube currency="RON" rate="4.4892"/>
    #             <Cube currency="SEK" rate="9.4883"/>
    #             <Cube currency="CHF" rate="1.2010"/>
    #             <Cube currency="NOK" rate="9.0605"/>
    #             <Cube currency="HRK" rate="7.6780"/>
    #             <Cube currency="RUB" rate="72.8910"/>
    #             <Cube currency="TRY" rate="2.7154"/>
    #             <Cube currency="AUD" rate="1.4506"/>
    #             <Cube currency="BRL" rate="3.1389"/>
    #             <Cube currency="CAD" rate="1.3963"/>
    #             <Cube currency="CNY" rate="7.3321"/>
    #             <Cube currency="HKD" rate="9.1593"/>
    #             <Cube currency="IDR" rate="14925.34"/>
    #             <Cube currency="ILS" rate="4.6614"/>
    #             <Cube currency="INR" rate="73.6233"/>
    #             <Cube currency="KRW" rate="1290.29"/>
    #             <Cube currency="MXN" rate="17.3190"/>
    #             <Cube currency="MYR" rate="4.2054"/>
    #             <Cube currency="NZD" rate="1.5115"/>
    #             <Cube currency="PHP" rate="53.090"/>
    #             <Cube currency="SGD" rate="1.5789"/>
    #             <Cube currency="THB" rate="38.846"/>
    #             <Cube currency="ZAR" rate="13.6655"/>
    #         </Cube>
    #     </Cube>
    # </gesmes:Envelope>

    # If we don't explicitly recode to UTF-8, ElementTree stupidly uses
    # ascii on Python 2.7
    envelope = ElementTree.fromstring(return_xml.encode('utf-8'))

    namespaces = {
        'gesmes': 'http://www.gesmes.org/xml/2002-08-01',
        'eurofxref': 'http://www.ecb.int/vocabulary/2002-08-01/eurofxref'
    }

    date_elements = envelope.findall('./eurofxref:Cube/eurofxref:Cube[@time]', namespaces)
    if not date_elements:
        # Fail loudly if the XML seems to have changed
        raise WebServiceError('Unable to find <Cube time=""> tag in ECB XML')

    date = date_elements[0].get('time')
    if not isinstance(date, str_cls):
        date = date.decode('utf-8')

    currency_elements = envelope.findall('./eurofxref:Cube/eurofxref:Cube/eurofxref:Cube[@currency][@rate]', namespaces)
    if not currency_elements:
        # Fail loudly if the XML seems to have changed
        raise WebServiceError('Unable to find <Cube currency="" rate=""> tags in ECB XML')

    rates = {
        'EUR': Decimal('1.0000')
    }

    applicable_currenties = {
        'BGN': True,
        'CZK': True,
        'DKK': True,
        'EUR': True,
        'GBP': True,
        'HRK': True,
        'HUF': True,
        'NOK': True,
        'PLN': True,
        'RON': True,
        'SEK': True,
        'USD': True
    }

    for currency_element in currency_elements:
        code = currency_element.attrib.get('currency')

        if code not in applicable_currenties:
            continue

        rate = currency_element.attrib.get('rate')
        rates[code] = Decimal(rate)

    return (date, rates)


def setup_xrates(base, rates):
    """
    If using the Python money package, this will set up the xrates exchange
    rate data.

    :param base:
        The string currency code to use as the base

    :param rates:
        A dict with keys that are string currency codes and values that are
        a Decimal of the exchange rate for that currency.
    """

    xrates.install('money.exchange.SimpleBackend')
    xrates.base = base
    for code, value in rates.items():
        xrates.setrate(code, value)


def format(amount, currency=None):
    """
    Formats a decimal or Money object into an unambiguous string representation
    for the purpose of invoices in English.

    :param amount:
        A Decimal or Money object

    :param currency:
        If the amount is a Decimal, the currency of the amount

    :return:
        A string representation of the amount in the currency
    """

    if currency is None and hasattr(amount, 'currency'):
        currency = amount.currency

    # Allow Money objects
    if not isinstance(amount, Decimal) and hasattr(amount, 'amount'):
        amount = amount.amount

    if not isinstance(currency, str_cls):
        raise ValueError('The currency specified is not a string')

    if currency not in FORMATTING_RULES:
        valid_currencies = sorted(FORMATTING_RULES.keys())
        formatted_currencies = ', '.join(valid_currencies)
        raise ValueError('The currency specified, "%s", is not a supported currency: %s' % (currency, formatted_currencies))

    if not isinstance(amount, Decimal):
        raise ValueError('The amount specified is not a Decimal')

    rules = FORMATTING_RULES[currency]

    format_string = ',.%sf' % rules['decimal_places']

    result = builtin_format(amount, format_string)

    result = result.replace(',', '_')
    result = result.replace('.', '|')

    result = result.replace('_', rules['thousands_separator'])
    result = result.replace('|', rules['decimal_mark'])

    if rules['symbol_first']:
        result = rules['symbol'] + result
    else:
        result = result + rules['symbol']

    return result


FORMATTING_RULES = {
    'BGN': {
        'symbol': ' Lev',
        'symbol_first': False,
        'decimal_mark': '.',
        'thousands_separator': ',',
        'decimal_places': 2
    },
    'CZK': {
        'symbol': ' Kč',
        'symbol_first': False,
        'decimal_mark': ',',
        'thousands_separator': '.',
        'decimal_places': 2
    },
    'DKK': {
        'symbol': ' Dkr',
        'symbol_first': False,
        'decimal_mark': ',',
        'thousands_separator': '.',
        'decimal_places': 2
    },
    'EUR': {
        'symbol': '€',
        'symbol_first': True,
        'decimal_mark': ',',
        'thousands_separator': '.',
        'decimal_places': 2
    },
    'GBP': {
        'symbol': '£',
        'symbol_first': True,
        'decimal_mark': '.',
        'thousands_separator': ',',
        'decimal_places': 2
    },
    'HRK': {
        'symbol': ' Kn',
        'symbol_first': False,
        'decimal_mark': ',',
        'thousands_separator': '.',
        'decimal_places': 2
    },
    'HUF': {
        'symbol': ' Ft',
        'symbol_first': False,
        'decimal_mark': ',',
        'thousands_separator': '.',
        'decimal_places': 2
    },
    'NOK': {
        'symbol': ' Nkr',
        'symbol_first': False,
        'decimal_mark': ',',
        'thousands_separator': '.',
        'decimal_places': 2
    },
    'PLN': {
        'symbol': ' Zł',
        'symbol_first': False,
        'decimal_mark': ',',
        'thousands_separator': ' ',
        'decimal_places': 2
    },
    'RON': {
        'symbol': ' Lei',
        'symbol_first': False,
        'decimal_mark': ',',
        'thousands_separator': '.',
        'decimal_places': 2
    },
    'SEK': {
        'symbol': ' Skr',
        'symbol_first': False,
        'decimal_mark': ',',
        'thousands_separator': ' ',
        'decimal_places': 2
    },
    'USD': {
        'symbol': '$',
        'symbol_first': True,
        'decimal_mark': '.',
        'thousands_separator': ',',
        'decimal_places': 2
    }
}



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from decimal import Decimal

try:
    # Python 2
    str_cls = unicode
except (NameError):
    # Python 3
    str_cls = str

from . import rates


def calculate_rate(country_code, exception_name):
    """
    Calculates the VAT rate for a customer based on their declared country
    and any declared exception information.

    :param country_code:
        The two-character country code where the user resides

    :param exception_name:
        The name of an exception for the country, as returned from
        vat_moss.declared_residence.options()

    :raises:
        ValueError - if country_code is not two characers, or exception_name is not None or a valid exception from options()

    :return:
        A tuple of (Decimal VAT rate, country_code, exception name [or None])
    """

    if not country_code or not isinstance(country_code, str_cls) or len(country_code) != 2:
        raise ValueError('Invalidly formatted country code')

    if exception_name and not isinstance(exception_name, str_cls):
        raise ValueError('Exception name is not None or a string')

    country_code = country_code.upper()

    if country_code not in rates.BY_COUNTRY:
        return (Decimal('0.0'), country_code, None)

    country_info = rates.BY_COUNTRY[country_code]

    if not exception_name:
        return (country_info['rate'], country_code, None)

    if exception_name not in country_info['exceptions']:
        raise ValueError('"%s" is not a valid exception for %s' % (exception_name, country_code))

    rate_info = country_info['exceptions'][exception_name]
    if isinstance(rate_info, Decimal):
        rate = rate_info
    else:
        # This allows handling the complex case of the UK RAF bases in Cyprus
        # that map to the standard country rate. The country code and exception
        # name need to be changed in addition to gettting a special rate.
        rate, country_code, exception_name = rate_info
    return (rate, country_code, exception_name)


def exceptions_by_country(country_code):
    """
    Returns a list of exception names for the given country

    :param country_code:
        The two-character country code for the user

    :raises:
        ValueError - if country_code is not two characers

    :return:
        A list of strings that are VAT exceptions for the country specified
    """

    if not country_code or not isinstance(country_code, str_cls) or len(country_code) != 2:
        raise ValueError('Invalidly formatted country code')

    country_code = country_code.upper()

    return EXCEPTIONS_BY_COUNTRY.get(country_code, [])


def options():
    """
    Return a sorted list of dicts, each containing the keys "name", "code" and
    "exceptions". These should be used to build a user interface for customers
    to declare their country of residence. If their declared country of
    residence includes any exceptions, the user must be presented with an option
    to select their residence as residing in an area with a VAT exception.

    The country codes and names are from ISO 3166-1.

    :return:
        A list of dicts objects each with the keys "name", "code" and
        "exceptions"
    """

    return [
        {
            'name': 'Afghanistan',
            'code': 'AF',
            'exceptions': []
        },
        {
            'name': 'Åland Islands',
            'code': 'AX',
            'exceptions': []
        },
        {
            'name': 'Albania',
            'code': 'AL',
            'exceptions': []
        },
        {
            'name': 'Algeria',
            'code': 'DZ',
            'exceptions': []
        },
        {
            'name': 'American Samoa',
            'code': 'AS',
            'exceptions': []
        },
        {
            'name': 'Andorra',
            'code': 'AD',
            'exceptions': []
        },
        {
            'name': 'Angola',
            'code': 'AO',
            'exceptions': []
        },
        {
            'name': 'Anguilla',
            'code': 'AI',
            'exceptions': []
        },
        {
            'name': 'Antarctica',
            'code': 'AQ',
            'exceptions': []
        },
        {
            'name': 'Antigua and Barbuda',
            'code': 'AG',
            'exceptions': []
        },
        {
            'name': 'Argentina',
            'code': 'AR',
            'exceptions': []
        },
        {
            'name': 'Armenia',
            'code': 'AM',
            'exceptions': []
        },
        {
            'name': 'Aruba',
            'code': 'AW',
            'exceptions': []
        },
        {
            'name': 'Australia',
            'code': 'AU',
            'exceptions': []
        },
        {
            'name': 'Austria',
            'code': 'AT',
            'exceptions': [
                'Jungholz',
                'Mittelberg'
            ]
        },
        {
            'name': 'Azerbaijan',
            'code': 'AZ',
            'exceptions': []
        },
        {
            'name': 'Bahamas',
            'code': 'BS',
            'exceptions': []
        },
        {
            'name': 'Bahrain',
            'code': 'BH',
            'exceptions': []
        },
        {
            'name': 'Bangladesh',
            'code': 'BD',
            'exceptions': []
        },
        {
            'name': 'Barbados',
            'code': 'BB',
            'exceptions': []
        },
        {
            'name': 'Belarus',
            'code': 'BY',
            'exceptions': []
        },
        {
            'name': 'Belgium',
            'code': 'BE',
            'exceptions': []
        },
        {
            'name': 'Belize',
            'code': 'BZ',
            'exceptions': []
        },
        {
            'name': 'Benin',
            'code': 'BJ',
            'exceptions': []
        },
        {
            'name': 'Bermuda',
            'code': 'BM',
            'exceptions': []
        },
        {
            'name': 'Bhutan',
            'code': 'BT',
            'exceptions': []
        },
        {
            'name': 'Bolivia, Plurinational State of',
            'code': 'BO',
            'exceptions': []
        },
        {
            'name': 'Bonaire, Sint Eustatius and Saba',
            'code': 'BQ',
            'exceptions': []
        },
        {
            'name': 'Bosnia and Herzegovina',
            'code': 'BA',
            'exceptions': []
        },
        {
            'name': 'Botswana',
            'code': 'BW',
            'exceptions': []
        },
        {
            'name': 'Bouvet Island',
            'code': 'BV',
            'exceptions': []
        },
        {
            'name': 'Brazil',
            'code': 'BR',
            'exceptions': []
        },
        {
            'name': 'British Indian Ocean Territory',
            'code': 'IO',
            'exceptions': []
        },
        {
            'name': 'Brunei Darussalam',
            'code': 'BN',
            'exceptions': []
        },
        {
            'name': 'Bulgaria',
            'code': 'BG',
            'exceptions': []
        },
        {
            'name': 'Burkina Faso',
            'code': 'BF',
            'exceptions': []
        },
        {
            'name': 'Burundi',
            'code': 'BI',
            'exceptions': []
        },
        {
            'name': 'Cambodia',
            'code': 'KH',
            'exceptions': []
        },
        {
            'name': 'Cameroon',
            'code': 'CM',
            'exceptions': []
        },
        {
            'name': 'Canada',
            'code': 'CA',
            'exceptions': []
        },
        {
            'name': 'Cabo Verde',
            'code': 'CV',
            'exceptions': []
        },
        {
            'name': 'Cayman Islands',
            'code': 'KY',
            'exceptions': []
        },
        {
            'name': 'Central African Republic',
            'code': 'CF',
            'exceptions': []
        },
        {
            'name': 'Chad',
            'code': 'TD',
            'exceptions': []
        },
        {
            'name': 'Chile',
            'code': 'CL',
            'exceptions': []
        },
        {
            'name': 'China',
            'code': 'CN',
            'exceptions': []
        },
        {
            'name': 'Christmas Island',
            'code': 'CX',
            'exceptions': []
        },
        {
            'name': 'Cocos (Keeling) Islands',
            'code': 'CC',
            'exceptions': []
        },
        {
            'name': 'Colombia',
            'code': 'CO',
            'exceptions': []
        },
        {
            'name': 'Comoros',
            'code': 'KM',
            'exceptions': []
        },
        {
            'name': 'Congo',
            'code': 'CG',
            'exceptions': []
        },
        {
            'name': 'Congo, the Democratic Republic of the',
            'code': 'CD',
            'exceptions': []
        },
        {
            'name': 'Cook Islands',
            'code': 'CK',
            'exceptions': []
        },
        {
            'name': 'Costa Rica',
            'code': 'CR',
            'exceptions': []
        },
        {
            'name': "Côte d'Ivoire",
            'code': 'CI',
            'exceptions': []
        },
        {
            'name': 'Croatia',
            'code': 'HR',
            'exceptions': []
        },
        {
            'name': 'Cuba',
            'code': 'CU',
            'exceptions': []
        },
        {
            'name': 'Curaçao',
            'code': 'CW',
            'exceptions': []
        },
        {
            'name': 'Cyprus',
            'code': 'CY',
            'exceptions': []
        },
        {
            'name': 'Czech Republic',
            'code': 'CZ',
            'exceptions': []
        },
        {
            'name': 'Denmark',
            'code': 'DK',
            'exceptions': []
        },
        {
            'name': 'Djibouti',
            'code': 'DJ',
            'exceptions': []
        },
        {
            'name': 'Dominica',
            'code': 'DM',
            'exceptions': []
        },
        {
            'name': 'Dominican Republic',
            'code': 'DO',
            'exceptions': []
        },
        {
            'name': 'Ecuador',
            'code': 'EC',
            'exceptions': []
        },
        {
            'name': 'Egypt',
            'code': 'EG',
            'exceptions': []
        },
        {
            'name': 'El Salvador',
            'code': 'SV',
            'exceptions': []
        },
        {
            'name': 'Equatorial Guinea',
            'code': 'GQ',
            'exceptions': []
        },
        {
            'name': 'Eritrea',
            'code': 'ER',
            'exceptions': []
        },
        {
            'name': 'Estonia',
            'code': 'EE',
            'exceptions': []
        },
        {
            'name': 'Ethiopia',
            'code': 'ET',
            'exceptions': []
        },
        {
            'name': 'Falkland Islands (Malvinas)',
            'code': 'FK',
            'exceptions': []
        },
        {
            'name': 'Faroe Islands',
            'code': 'FO',
            'exceptions': []
        },
        {
            'name': 'Fiji',
            'code': 'FJ',
            'exceptions': []
        },
        {
            'name': 'Finland',
            'code': 'FI',
            'exceptions': []
        },
        {
            'name': 'France',
            'code': 'FR',
            'exceptions': []
        },
        {
            'name': 'French Guiana',
            'code': 'GF',
            'exceptions': []
        },
        {
            'name': 'French Polynesia',
            'code': 'PF',
            'exceptions': []
        },
        {
            'name': 'French Southern Territories',
            'code': 'TF',
            'exceptions': []
        },
        {
            'name': 'Gabon',
            'code': 'GA',
            'exceptions': []
        },
        {
            'name': 'Gambia',
            'code': 'GM',
            'exceptions': []
        },
        {
            'name': 'Georgia',
            'code': 'GE',
            'exceptions': []
        },
        {
            'name': 'Germany',
            'code': 'DE',
            'exceptions': [
                'Büsingen am Hochrhein',
                'Heligoland'
            ]
        },
        {
            'name': 'Ghana',
            'code': 'GH',
            'exceptions': []
        },
        {
            'name': 'Gibraltar',
            'code': 'GI',
            'exceptions': []
        },
        {
            'name': 'Greece',
            'code': 'GR',
            'exceptions': [
                'Mount Athos'
            ]
        },
        {
            'name': 'Greenland',
            'code': 'GL',
            'exceptions': []
        },
        {
            'name': 'Grenada',
            'code': 'GD',
            'exceptions': []
        },
        {
            'name': 'Guadeloupe',
            'code': 'GP',
            'exceptions': []
        },
        {
            'name': 'Guam',
            'code': 'GU',
            'exceptions': []
        },
        {
            'name': 'Guatemala',
            'code': 'GT',
            'exceptions': []
        },
        {
            'name': 'Guernsey',
            'code': 'GG',
            'exceptions': []
        },
        {
            'name': 'Guinea',
            'code': 'GN',
            'exceptions': []
        },
        {
            'name': 'Guinea-Bissau',
            'code': 'GW',
            'exceptions': []
        },
        {
            'name': 'Guyana',
            'code': 'GY',
            'exceptions': []
        },
        {
            'name': 'Haiti',
            'code': 'HT',
            'exceptions': []
        },
        {
            'name': 'Heard Island and McDonald Islands',
            'code': 'HM',
            'exceptions': []
        },
        {
            'name': 'Holy See (Vatican City State)',
            'code': 'VA',
            'exceptions': []
        },
        {
            'name': 'Honduras',
            'code': 'HN',
            'exceptions': []
        },
        {
            'name': 'Hong Kong',
            'code': 'HK',
            'exceptions': []
        },
        {
            'name': 'Hungary',
            'code': 'HU',
            'exceptions': []
        },
        {
            'name': 'Iceland',
            'code': 'IS',
            'exceptions': []
        },
        {
            'name': 'India',
            'code': 'IN',
            'exceptions': []
        },
        {
            'name': 'Indonesia',
            'code': 'ID',
            'exceptions': []
        },
        {
            'name': 'Iran, Islamic Republic of',
            'code': 'IR',
            'exceptions': []
        },
        {
            'name': 'Iraq',
            'code': 'IQ',
            'exceptions': []
        },
        {
            'name': 'Ireland',
            'code': 'IE',
            'exceptions': []
        },
        {
            'name': 'Isle of Man',
            'code': 'IM',
            'exceptions': []
        },
        {
            'name': 'Israel',
            'code': 'IL',
            'exceptions': []
        },
        {
            'name': 'Italy',
            'code': 'IT',
            'exceptions': [
                "Campione d'Italia",
                'Livigno'
            ]
        },
        {
            'name': 'Jamaica',
            'code': 'JM',
            'exceptions': []
        },
        {
            'name': 'Japan',
            'code': 'JP',
            'exceptions': []
        },
        {
            'name': 'Jersey',
            'code': 'JE',
            'exceptions': []
        },
        {
            'name': 'Jordan',
            'code': 'JO',
            'exceptions': []
        },
        {
            'name': 'Kazakhstan',
            'code': 'KZ',
            'exceptions': []
        },
        {
            'name': 'Kenya',
            'code': 'KE',
            'exceptions': []
        },
        {
            'name': 'Kiribati',
            'code': 'KI',
            'exceptions': []
        },
        {
            'name': "Korea, Democratic People's Republic of",
            'code': 'KP',
            'exceptions': []
        },
        {
            'name': 'Korea, Republic of',
            'code': 'KR',
            'exceptions': []
        },
        {
            'name': 'Kosovo, Republic of',
            'code': 'XK',
            'exceptions': []
        },
        {
            'name': 'Kuwait',
            'code': 'KW',
            'exceptions': []
        },
        {
            'name': 'Kyrgyzstan',
            'code': 'KG',
            'exceptions': []
        },
        {
            'name': "Lao People's Democratic Republic",
            'code': 'LA',
            'exceptions': []
        },
        {
            'name': 'Latvia',
            'code': 'LV',
            'exceptions': []
        },
        {
            'name': 'Lebanon',
            'code': 'LB',
            'exceptions': []
        },
        {
            'name': 'Lesotho',
            'code': 'LS',
            'exceptions': []
        },
        {
            'name': 'Liberia',
            'code': 'LR',
            'exceptions': []
        },
        {
            'name': 'Libya',
            'code': 'LY',
            'exceptions': []
        },
        {
            'name': 'Liechtenstein',
            'code': 'LI',
            'exceptions': []
        },
        {
            'name': 'Lithuania',
            'code': 'LT',
            'exceptions': []
        },
        {
            'name': 'Luxembourg',
            'code': 'LU',
            'exceptions': []
        },
        {
            'name': 'Macao',
            'code': 'MO',
            'exceptions': []
        },
        {
            'name': 'Macedonia, the former Yugoslav Republic of',
            'code': 'MK',
            'exceptions': []
        },
        {
            'name': 'Madagascar',
            'code': 'MG',
            'exceptions': []
        },
        {
            'name': 'Malawi',
            'code': 'MW',
            'exceptions': []
        },
        {
            'name': 'Malaysia',
            'code': 'MY',
            'exceptions': []
        },
        {
            'name': 'Maldives',
            'code': 'MV',
            'exceptions': []
        },
        {
            'name': 'Mali',
            'code': 'ML',
            'exceptions': []
        },
        {
            'name': 'Malta',
            'code': 'MT',
            'exceptions': []
        },
        {
            'name': 'Marshall Islands',
            'code': 'MH',
            'exceptions': []
        },
        {
            'name': 'Martinique',
            'code': 'MQ',
            'exceptions': []
        },
        {
            'name': 'Mauritania',
            'code': 'MR',
            'exceptions': []
        },
        {
            'name': 'Mauritius',
            'code': 'MU',
            'exceptions': []
        },
        {
            'name': 'Mayotte',
            'code': 'YT',
            'exceptions': []
        },
        {
            'name': 'Mexico',
            'code': 'MX',
            'exceptions': []
        },
        {
            'name': 'Micronesia, Federated States of',
            'code': 'FM',
            'exceptions': []
        },
        {
            'name': 'Moldova, Republic of',
            'code': 'MD',
            'exceptions': []
        },
        {
            'name': 'Monaco',
            'code': 'MC',
            'exceptions': []
        },
        {
            'name': 'Mongolia',
            'code': 'MN',
            'exceptions': []
        },
        {
            'name': 'Montenegro',
            'code': 'ME',
            'exceptions': []
        },
        {
            'name': 'Montserrat',
            'code': 'MS',
            'exceptions': []
        },
        {
            'name': 'Morocco',
            'code': 'MA',
            'exceptions': []
        },
        {
            'name': 'Mozambique',
            'code': 'MZ',
            'exceptions': []
        },
        {
            'name': 'Myanmar',
            'code': 'MM',
            'exceptions': []
        },
        {
            'name': 'Namibia',
            'code': 'NA',
            'exceptions': []
        },
        {
            'name': 'Nauru',
            'code': 'NR',
            'exceptions': []
        },
        {
            'name': 'Nepal',
            'code': 'NP',
            'exceptions': []
        },
        {
            'name': 'Netherlands',
            'code': 'NL',
            'exceptions': []
        },
        {
            'name': 'New Caledonia',
            'code': 'NC',
            'exceptions': []
        },
        {
            'name': 'New Zealand',
            'code': 'NZ',
            'exceptions': []
        },
        {
            'name': 'Nicaragua',
            'code': 'NI',
            'exceptions': []
        },
        {
            'name': 'Niger',
            'code': 'NE',
            'exceptions': []
        },
        {
            'name': 'Nigeria',
            'code': 'NG',
            'exceptions': []
        },
        {
            'name': 'Niue',
            'code': 'NU',
            'exceptions': []
        },
        {
            'name': 'Norfolk Island',
            'code': 'NF',
            'exceptions': []
        },
        {
            'name': 'Northern Mariana Islands',
            'code': 'MP',
            'exceptions': []
        },
        {
            'name': 'Norway',
            'code': 'NO',
            'exceptions': []
        },
        {
            'name': 'Oman',
            'code': 'OM',
            'exceptions': []
        },
        {
            'name': 'Pakistan',
            'code': 'PK',
            'exceptions': []
        },
        {
            'name': 'Palau',
            'code': 'PW',
            'exceptions': []
        },
        {
            'name': 'Palestine, State of',
            'code': 'PS',
            'exceptions': []
        },
        {
            'name': 'Panama',
            'code': 'PA',
            'exceptions': []
        },
        {
            'name': 'Papua New Guinea',
            'code': 'PG',
            'exceptions': []
        },
        {
            'name': 'Paraguay',
            'code': 'PY',
            'exceptions': []
        },
        {
            'name': 'Peru',
            'code': 'PE',
            'exceptions': []
        },
        {
            'name': 'Philippines',
            'code': 'PH',
            'exceptions': []
        },
        {
            'name': 'Pitcairn',
            'code': 'PN',
            'exceptions': []
        },
        {
            'name': 'Poland',
            'code': 'PL',
            'exceptions': []
        },
        {
            'name': 'Portugal',
            'code': 'PT',
            'exceptions': [
                'Azores',
                'Madeira'
            ]
        },
        {
            'name': 'Puerto Rico',
            'code': 'PR',
            'exceptions': []
        },
        {
            'name': 'Qatar',
            'code': 'QA',
            'exceptions': []
        },
        {
            'name': 'Réunion',
            'code': 'RE',
            'exceptions': []
        },
        {
            'name': 'Romania',
            'code': 'RO',
            'exceptions': []
        },
        {
            'name': 'Russian Federation',
            'code': 'RU',
            'exceptions': []
        },
        {
            'name': 'Rwanda',
            'code': 'RW',
            'exceptions': []
        },
        {
            'name': 'Saint Barthélemy',
            'code': 'BL',
            'exceptions': []
        },
        {
            'name': 'Saint Helena, Ascension and Tristan da Cunha',
            'code': 'SH',
            'exceptions': []
        },
        {
            'name': 'Saint Kitts and Nevis',
            'code': 'KN',
            'exceptions': []
        },
        {
            'name': 'Saint Lucia',
            'code': 'LC',
            'exceptions': []
        },
        {
            'name': 'Saint Martin (French part)',
            'code': 'MF',
            'exceptions': []
        },
        {
            'name': 'Saint Pierre and Miquelon',
            'code': 'PM',
            'exceptions': []
        },
        {
            'name': 'Saint Vincent and the Grenadines',
            'code': 'VC',
            'exceptions': []
        },
        {
            'name': 'Samoa',
            'code': 'WS',
            'exceptions': []
        },
        {
            'name': 'San Marino',
            'code': 'SM',
            'exceptions': []
        },
        {
            'name': 'Sao Tome and Principe',
            'code': 'ST',
            'exceptions': []
        },
        {
            'name': 'Saudi Arabia',
            'code': 'SA',
            'exceptions': []
        },
        {
            'name': 'Senegal',
            'code': 'SN',
            'exceptions': []
        },
        {
            'name': 'Serbia',
            'code': 'RS',
            'exceptions': []
        },
        {
            'name': 'Seychelles',
            'code': 'SC',
            'exceptions': []
        },
        {
            'name': 'Sierra Leone',
            'code': 'SL',
            'exceptions': []
        },
        {
            'name': 'Singapore',
            'code': 'SG',
            'exceptions': []
        },
        {
            'name': 'Sint Maarten (Dutch part)',
            'code': 'SX',
            'exceptions': []
        },
        {
            'name': 'Slovakia',
            'code': 'SK',
            'exceptions': []
        },
        {
            'name': 'Slovenia',
            'code': 'SI',
            'exceptions': []
        },
        {
            'name': 'Solomon Islands',
            'code': 'SB',
            'exceptions': []
        },
        {
            'name': 'Somalia',
            'code': 'SO',
            'exceptions': []
        },
        {
            'name': 'South Africa',
            'code': 'ZA',
            'exceptions': []
        },
        {
            'name': 'South Georgia and the South Sandwich Islands',
            'code': 'GS',
            'exceptions': []
        },
        {
            'name': 'South Sudan',
            'code': 'SS',
            'exceptions': []
        },
        {
            'name': 'Spain',
            'code': 'ES',
            'exceptions': [
                'Canary Islands',
                'Ceuta',
                'Melilla'
            ]
        },
        {
            'name': 'Sri Lanka',
            'code': 'LK',
            'exceptions': []
        },
        {
            'name': 'Sudan',
            'code': 'SD',
            'exceptions': []
        },
        {
            'name': 'Suriname',
            'code': 'SR',
            'exceptions': []
        },
        {
            'name': 'Svalbard and Jan Mayen',
            'code': 'SJ',
            'exceptions': []
        },
        {
            'name': 'Swaziland',
            'code': 'SZ',
            'exceptions': []
        },
        {
            'name': 'Sweden',
            'code': 'SE',
            'exceptions': []
        },
        {
            'name': 'Switzerland',
            'code': 'CH',
            'exceptions': []
        },
        {
            'name': 'Syrian Arab Republic',
            'code': 'SY',
            'exceptions': []
        },
        {
            'name': 'Taiwan, Province of China',
            'code': 'TW',
            'exceptions': []
        },
        {
            'name': 'Tajikistan',
            'code': 'TJ',
            'exceptions': []
        },
        {
            'name': 'Tanzania, United Republic of',
            'code': 'TZ',
            'exceptions': []
        },
        {
            'name': 'Thailand',
            'code': 'TH',
            'exceptions': []
        },
        {
            'name': 'Timor-Leste',
            'code': 'TL',
            'exceptions': []
        },
        {
            'name': 'Togo',
            'code': 'TG',
            'exceptions': []
        },
        {
            'name': 'Tokelau',
            'code': 'TK',
            'exceptions': []
        },
        {
            'name': 'Tonga',
            'code': 'TO',
            'exceptions': []
        },
        {
            'name': 'Trinidad and Tobago',
            'code': 'TT',
            'exceptions': []
        },
        {
            'name': 'Tunisia',
            'code': 'TN',
            'exceptions': []
        },
        {
            'name': 'Turkey',
            'code': 'TR',
            'exceptions': []
        },
        {
            'name': 'Turkmenistan',
            'code': 'TM',
            'exceptions': []
        },
        {
            'name': 'Turks and Caicos Islands',
            'code': 'TC',
            'exceptions': []
        },
        {
            'name': 'Tuvalu',
            'code': 'TV',
            'exceptions': []
        },
        {
            'name': 'Uganda',
            'code': 'UG',
            'exceptions': []
        },
        {
            'name': 'Ukraine',
            'code': 'UA',
            'exceptions': []
        },
        {
            'name': 'United Arab Emirates',
            'code': 'AE',
            'exceptions': []
        },
        {
            'name': 'United Kingdom',
            'code': 'GB',
            'exceptions': [
                'Akrotiri',
                'Dhekelia'
            ]
        },
        {
            'name': 'United States',
            'code': 'US',
            'exceptions': []
        },
        {
            'name': 'United States Minor Outlying Islands',
            'code': 'UM',
            'exceptions': []
        },
        {
            'name': 'Uruguay',
            'code': 'UY',
            'exceptions': []
        },
        {
            'name': 'Uzbekistan',
            'code': 'UZ',
            'exceptions': []
        },
        {
            'name': 'Vanuatu',
            'code': 'VU',
            'exceptions': []
        },
        {
            'name': 'Venezuela, Bolivarian Republic of',
            'code': 'VE',
            'exceptions': []
        },
        {
            'name': 'Viet Nam',
            'code': 'VN',
            'exceptions': []
        },
        {
            'name': 'Virgin Islands, British',
            'code': 'VG',
            'exceptions': []
        },
        {
            'name': 'Virgin Islands, U.S.',
            'code': 'VI',
            'exceptions': []
        },
        {
            'name': 'Wallis and Futuna',
            'code': 'WF',
            'exceptions': []
        },
        {
            'name': 'Western Sahara',
            'code': 'EH',
            'exceptions': []
        },
        {
            'name': 'Yemen',
            'code': 'YE',
            'exceptions': []
        },
        {
            'name': 'Zambia',
            'code': 'ZM',
            'exceptions': []
        },
        {
            'name': 'Zimbabwe',
            'code': 'ZW',
            'exceptions': []
        }
    ]


# The valid exception names, listed by country
EXCEPTIONS_BY_COUNTRY = {
    'AT': ['Jungholz', 'Mittelberg'],
    'DE': ['Büsingen am Hochrhein', 'Heligoland'],
    'ES': ['Canary Islands', 'Ceuta', 'Melilla'],
    'GB': ['Akrotiri', 'Dhekelia'],
    'GR': ['Mount Athos'],
    'IT': ["Campione d'Italia", 'Livigno'],
    'PT': ['Azores', 'Madeira']
}



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import re
import json
from xml.etree import ElementTree
import cgi

try:
    # Python 3
    from urllib.request import Request, urlopen
    from urllib.error import HTTPError
    str_cls = str
except (ImportError):
    # Python 2
    from urllib2 import Request, urlopen, HTTPError
    str_cls = unicode

from .errors import InvalidError, WebServiceError, WebServiceUnavailableError


def normalize(vat_id):
    """
    Accepts a VAT ID and normaizes it, getting rid of spaces, periods, dashes
    etc and converting it to upper case.

    :param vat_id:
        The VAT ID to check. Allows "GR" prefix for Greece, even though it
        should be "EL".

    :raises:
        ValueError - If the is not a string or is not in the format of two characters plus an identifier

    :return:
        None if the VAT ID is blank or not for an EU country or Norway
        Otherwise a normalized string containing the VAT ID
    """

    if not vat_id:
        return None

    if not isinstance(vat_id, str_cls):
        raise ValueError('VAT ID is not a string')

    if len(vat_id) < 3:
        raise ValueError('VAT ID must be at least three character long')

    # Normalize the ID for simpler regexes
    vat_id = re.sub('\\s+', '', vat_id)
    vat_id = vat_id.replace('-', '')
    vat_id = vat_id.replace('.', '')
    vat_id = vat_id.upper()

    country_prefix = vat_id[0:2]

    # Fix people using GR prefix for Greece
    if country_prefix == 'GR':
        vat_id = 'EL' + vat_id[2:]
        country_prefix = 'EL'

    if country_prefix not in ID_PATTERNS:
        return None

    return vat_id


def validate(vat_id):
    """
    Runs some basic checks to ensure a VAT ID looks properly formatted. If so,
    checks it against the VIES system for EU VAT IDs or data.brreg.no for
    Norwegian VAT ID.

    :param vat_id:
        The VAT ID to check. Allows "GR" prefix for Greece, even though it
        should be "EL".

    :raises:
        ValueError - If the is not a string or is not in the format of two characters plus an identifier
        InvalidError - If the VAT ID is not valid
        WebServiceUnavailableError - If the VIES VAT ID service is unable to process the request - this is fairly common
        WebServiceError - If there was an error parsing the response from the server - usually this means something changed in the webservice
        urllib.error.URLError/urllib2.URLError - If there is an issue communicating with VIES or data.brreg.no

    :return:
        None if the VAT ID is blank or not for an EU country or Norway
        A tuple of (two-character country code, normalized VAT id, company name) if valid
    """

    vat_id = normalize(vat_id)

    if not vat_id:
        return vat_id

    country_prefix = vat_id[0:2]

    number = vat_id[2:]

    if not re.match(ID_PATTERNS[country_prefix]['regex'], number):
        raise InvalidError('VAT ID does not appear to be properly formatted for %s' % country_prefix)

    if country_prefix == 'NO':
        organization_number = number.replace('MVA', '')
        validation_url = 'http://data.brreg.no/enhetsregisteret/enhet/%s.json' % organization_number

        try:
            response = urlopen(validation_url)
            _, params = cgi.parse_header(response.headers['Content-Type'])
            if 'charset' in params:
                encoding = params['charset']
            else:
                encoding = 'utf-8'

            return_json = response.read().decode(encoding)

            # Example response:
            #
            # {
            #     "organisasjonsnummer": 974760673,
            #     "navn": "REGISTERENHETEN I BRØNNØYSUND",
            #     "registreringsdatoEnhetsregisteret": "1995-08-09",
            #     "organisasjonsform": "ORGL",
            #     "hjemmeside": "www.brreg.no",
            #     "registrertIFrivillighetsregisteret": "N",
            #     "registrertIMvaregisteret": "N",
            #     "registrertIForetaksregisteret": "N",
            #     "registrertIStiftelsesregisteret": "N",
            #     "antallAnsatte": 562,
            #     "institusjonellSektorkode": {
            #         "kode": "6100",
            #         "beskrivelse": "Statsforvaltningen"
            #     },
            #     "naeringskode1": {
            #         "kode": "84.110",
            #         "beskrivelse": "Generell offentlig administrasjon"
            #     },
            #     "postadresse": {
            #         "adresse": "Postboks 900",
            #         "postnummer": "8910",
            #         "poststed": "BRØNNØYSUND",
            #         "kommunenummer": "1813",
            #         "kommune": "BRØNNØY",
            #         "landkode": "NO",
            #         "land": "Norge"
            #     },
            #     "forretningsadresse": {
            #         "adresse": "Havnegata 48",
            #         "postnummer": "8900",
            #         "poststed": "BRØNNØYSUND",
            #         "kommunenummer": "1813",
            #         "kommune": "BRØNNØY",
            #         "landkode": "NO",
            #         "land": "Norge"
            #     },
            #     "konkurs": "N",
            #     "underAvvikling": "N",
            #     "underTvangsavviklingEllerTvangsopplosning": "N",
            #     "overordnetEnhet": 912660680,
            #     "links": [
            #         {
            #             "rel": "self",
            #             "href": "http://data.brreg.no/enhetsregisteret/enhet/974760673"
            #         },
            #         {
            #             "rel": "overordnetEnhet",
            #             "href": "http://data.brreg.no/enhetsregisteret/enhet/912660680"
            #         }
            #     ]
            # }

            info = json.loads(return_json)

            # This should never happen, but keeping it incase the API is changed
            if 'organisasjonsnummer' not in info or info['organisasjonsnummer'] != int(organization_number):
                raise WebServiceError('No or different value for the "organisasjonsnummer" key in response from data.brreg.no')

            company_name = info['organisasjonsnummer']

        except (HTTPError) as e:
            # If a number is invalid, we get a 404
            if e.code == 404:
                raise InvalidError('VAT ID is invalid')

            # If we get anything but a 404 we want the exception to be recorded
            raise

    # EU countries
    else:
        post_data = '''
            <soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/" xmlns:urn="urn:ec.europa.eu:taxud:vies:services:checkVat:types">
               <soapenv:Header/>
               <soapenv:Body>
                  <urn:checkVat>
                     <urn:countryCode>%s</urn:countryCode>
                     <urn:vatNumber>%s</urn:vatNumber>
                  </urn:checkVat>
               </soapenv:Body>
            </soapenv:Envelope>
        ''' % (country_prefix, number)

        request = Request('http://ec.europa.eu/taxation_customs/vies/services/checkVatService')
        request.add_header('Content-Type', 'application/x-www-form-urlencoded; charset=utf-8')

        try:
            response = urlopen(request, post_data.encode('utf-8'))
        except (HTTPError) as e:
            # If one of the country VAT ID services is down, we get a 500
            if e.code == 500:
                raise WebServiceUnavailableError('VAT ID validation is not currently available')

            # If we get anything but a 500 we want the exception to be recorded
            raise

        _, params = cgi.parse_header(response.headers['Content-Type'])
        if 'charset' in params:
            encoding = params['charset']
        else:
            encoding = 'utf-8'

        return_xml = response.read().decode(encoding)

        # Example response:
        #
        # <soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/">
        #    <soap:Body>
        #       <checkVatResponse xmlns="urn:ec.europa.eu:taxud:vies:services:checkVat:types">
        #          <countryCode>GB</countryCode>
        #          <vatNumber>GD001</vatNumber>
        #          <requestDate>2014-12-17+01:00</requestDate>
        #          <valid>true</valid>
        #          <name>MINISTRY OF AGRICULTURE FISHERIES &amp; FOOD</name>
        #          <address>ROOM C206
        # GOVERNMENT BUILDINGS
        # EPSOM ROAD
        # GUILDFORD
        # SURREY
        # GU1 2LD</address>
        #       </checkVatResponse>
        #    </soap:Body>
        # </soap:Envelope>

        try:
            # If we don't explicitly recode to UTF-8, ElementTree stupidly uses
            # ascii on Python 2.7
            envelope = ElementTree.fromstring(return_xml.encode('utf-8'))
        except (ElementTree.ParseError):
            raise WebServiceError('Unable to parse response from VIES')

        namespaces = {
            'soap': 'http://schemas.xmlsoap.org/soap/envelope/',
            'vat':  'urn:ec.europa.eu:taxud:vies:services:checkVat:types'
        }
        valid_elements = envelope.findall('./soap:Body/vat:checkVatResponse/vat:valid', namespaces)
        if not valid_elements:
            # Fail loudly if the XML seems to have changed
            raise WebServiceError('Unable to find <valid> tag in response from VIES')

        name_elements = envelope.findall('./soap:Body/vat:checkVatResponse/vat:name', namespaces)
        if not name_elements:
            # Fail loudly if the XML seems to have changed
            raise WebServiceError('Unable to find <name> tag in response from VIES')

        if valid_elements[0].text.lower() != 'true':
            raise InvalidError('VAT ID is invalid')

        company_name = name_elements[0].text

    return (ID_PATTERNS[country_prefix]['country_code'], vat_id, company_name)


# Patterns generated by consulting the following URLs:
#
#  - http://en.wikipedia.org/wiki/VAT_identification_number
#  - http://ec.europa.eu/taxation_customs/vies/faq.html
#  - http://www.skatteetaten.no/en/International-pages/Felles-innhold-benyttes-i-flere-malgrupper/Brochure/Guide-to-value-added-tax-in-Norway/?chapter=7159
ID_PATTERNS = {
    'AT': {  # Austria
        'regex': '^U\\d{8}$',
        'country_code': 'AT'
    },
    'BE': {  # Belgium
        'regex': '^(1|0?)\\d{9}$',
        'country_code': 'BE'
    },
    'BG': {  # Bulgaria
        'regex': '^\\d{9,10}$',
        'country_code': 'BG'
    },
    'CY': {  # Cyprus
        'regex': '^\\d{8}[A-Z]$',
        'country_code': 'CY'
    },
    'CZ': {  # Czech Republic
        'regex': '^\\d{8,10}$',
        'country_code': 'CZ'
    },
    'DE': {  # Germany
        'regex': '^\\d{9}$',
        'country_code': 'DE'
    },
    'DK': {  # Denmark
        'regex': '^\\d{8}$',
        'country_code': 'DK'
    },
    'EE': {  # Estonia
        'regex': '^\\d{9}$',
        'country_code': 'EE'
    },
    'EL': {  # Greece
        'regex': '^\\d{9}$',
        'country_code': 'GR'
    },
    'ES': {  # Spain
        'regex': '^[A-Z0-9]\\d{7}[A-Z0-9]$',
        'country_code': 'ES'
    },
    'FI': {  # Finland
        'regex': '^\\d{8}$',
        'country_code': 'FI'
    },
    'FR': {  # France
        'regex': '^[A-Z0-9]{2}\\d{9}$',
        'country_code': 'FR'
    },
    'GB': {  # United Kingdom
        'regex': '^(GD\\d{3}|HA\\d{3}|\\d{9}|\\d{12})$',
        'country_code': 'GB'
    },
    'HR': {  # Croatia
        'regex': '^\\d{11}$',
        'country_code': 'HR'
    },
    'HU': {  # Hungary
        'regex': '^\\d{8}$',
        'country_code': 'HU'
    },
    'IE': {  # Ireland
        'regex': '^(\\d{7}[A-Z]{1,2}|\\d[A-Z+*]\\d{5}[A-Z])$',
        'country_code': 'IE'
    },
    'IT': {  # Italy
        'regex': '^\\d{11}$',
        'country_code': 'IT'
    },
    'LT': {  # Lithuania
        'regex': '^(\\d{9}|\\d{12})$',
        'country_code': 'LT'
    },
    'LU': {  # Luxembourg
        'regex': '^\\d{8}$',
        'country_code': 'LU'
    },
    'LV': {  # Latvia
        'regex': '^\\d{11}$',
        'country_code': 'LV'
    },
    'MT': {  # Malta
        'regex': '^\\d{8}$',
        'country_code': 'MT'
    },
    'NL': {  # Netherlands
        'regex': '^\\d{9}B\\d{2}$',
        'country_code': 'NL'
    },
    'NO': {  # Norway
        'regex': '^\\d{9}MVA$',
        'country_code': 'NO'
    },
    'PL': {  # Poland
        'regex': '^\\d{10}$',
        'country_code': 'PL'
    },
    'PT': {  # Portugal
        'regex': '^\\d{9}$',
        'country_code': 'PT'
    },
    'RO': {  # Romania
        'regex': '^\\d{2,10}$',
        'country_code': 'RO'
    },
    'SE': {  # Sweden
        'regex': '^\\d{12}$',
        'country_code': 'SE'
    },
    'SI': {  # Slovenia
        'regex': '^\\d{8}$',
        'country_code': 'SI'
    },
    'SK': {  # Slovakia
        'regex': '^\\d{10}$',
        'country_code': 'SK'
    },
}



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import re
from decimal import Decimal

try:
    # Python 2
    str_cls = unicode
except (NameError):
    # Python 3
    str_cls = str

from . import rates


def calculate_rate(country_code, postal_code, city):
    """
    Calculates the VAT rate that should be collected based on address
    information provided

    :param country_code:
        The two-character country code

    :param postal_code:
        The postal code for the user

    :param city:
        The city name for the user

    :raises:
        ValueError - If country code is not two characers, or postal_code or city are not strings. postal_code may be None or blank string for countries without postal codes.

    :return:
        A tuple of (Decimal percentage rate, country code, exception name [or None])
    """

    if not country_code or not isinstance(country_code, str_cls):
        raise ValueError('Invalidly formatted country code')

    country_code = country_code.strip()
    if len(country_code) != 2:
        raise ValueError('Invalidly formatted country code')

    country_code = country_code.upper()

    if country_code not in COUNTRIES_WITHOUT_POSTAL_CODES:
        if not postal_code or not isinstance(postal_code, str_cls):
            raise ValueError('Postal code is not a string')

    if not city or not isinstance(city, str_cls):
        raise ValueError('City is not a string')

    if isinstance(postal_code, str_cls):
        postal_code = re.sub('\\s+', '', postal_code)
        postal_code = postal_code.upper()

        # Remove the common european practice of adding the country code
        # to the beginning of a postal code, followed by a dash
        if len(postal_code) > 3 and postal_code[0:3] == country_code + '-':
            postal_code = postal_code[3:]

        postal_code = postal_code.replace('-', '')

    city = city.lower().strip()

    if country_code not in rates.BY_COUNTRY and country_code not in POSTAL_CODE_EXCEPTIONS:
        return (Decimal('0.0'), country_code, None)

    country_default = rates.BY_COUNTRY.get(country_code, {'rate': Decimal('0.0')})['rate']

    if country_code not in POSTAL_CODE_EXCEPTIONS:
        return (country_default, country_code, None)

    exceptions = POSTAL_CODE_EXCEPTIONS[country_code]
    for matcher in exceptions:
        # Postal code-only match
        if isinstance(matcher, str_cls):
            postal_regex = matcher
            city_regex = None
        else:
            postal_regex, city_regex = matcher

        if not re.match(postal_regex, postal_code):
            continue

        if city_regex and not re.search(city_regex, city):
            continue

        mapped_country = exceptions[matcher]['country_code']

        # There is at least one entry where we map to a different country,
        # but are not mapping to an exception
        if 'name' not in exceptions[matcher]:
            country_code = mapped_country
            country_default = rates.BY_COUNTRY[country_code]['rate']
            break

        mapped_name = exceptions[matcher]['name']

        rate = rates.BY_COUNTRY[mapped_country]['exceptions'][mapped_name]
        return (rate, mapped_country, mapped_name)

    return (country_default, country_code, None)


# A dictionary of countries, each being dictionary with keys that are either
# a string postal code regex, or a tuple of postal code regex and city name
# regex.
#
# There is a country_code value because some jurisdictions have post offices
# through multiple countries.
#
# These should only be used with billing addresses.
POSTAL_CODE_EXCEPTIONS = {
    'AT': {
        '^6691$': {
            'country_code': 'AT',
            'name': 'Jungholz'
        },
        ('^699[123]$', '\\bmittelberg\\b'): {
            'country_code': 'AT',
            'name': 'Mittelberg'
        }
    },
    'CH': {
        '^8238$': {
            'country_code': 'DE',
            'name': 'Büsingen am Hochrhein'
        },
        '^6911$': {
            'country_code': 'IT',
            'name': "Campione d'Italia"
        },
        # The Italian city of Domodossola has a Swiss post office also
        '^3907$': {
            'country_code': 'IT'
        }
    },
    'DE': {
        '^87491$': {
            'country_code': 'AT',
            'name': 'Jungholz'
        },
        ('^8756[789]$', '\\bmittelberg\\b'): {
            'country_code': 'AT',
            'name': 'Mittelberg'
        },
        '^78266$': {
            'country_code': 'DE',
            'name': 'Büsingen am Hochrhein'
        },
        '^27498$': {
            'country_code': 'DE',
            'name': 'Heligoland'
        }
    },
    'ES': {
        '^(5100[1-5]|5107[0-1]|51081)$': {
            'country_code': 'ES',
            'name': 'Ceuta'
        },
        '^(5200[0-6]|5207[0-1]|52081)$': {
            'country_code': 'ES',
            'name': 'Melilla'
        },
        '^(35\\d{3}|38\\d{3})$': {
            'country_code': 'ES',
            'name': 'Canary Islands'
        }
    },
    # The UK RAF bases in Cyprus are taxed at the Cyprus rate
    'GB': {
        # Akrotiri
        '^BFPO57|BF12AT$': {
            'country_code': 'CY'
        },
        # Dhekelia
        '^BFPO58|BF12AU$': {
            'country_code': 'CY'
        }
    },
    'GR': {
        '^63086$': {
            'country_code': 'GR',
            'name': 'Mount Athos'
        }
    },
    'IT': {
        ('^22060$', '\\bcampione\\b'): {
            'country_code': 'IT',
            'name': "Campione d'Italia"
        },
        ('^23030$', '\\blivigno\\b'): {
            'country_code': 'IT',
            'name': 'Livigno'
        }
    },
    'PT': {
        '^9[0-4]\\d{2,}$': {
            'country_code': 'PT',
            'name': 'Madeira'
        },
        '^9[5-9]\\d{2,}$': {
            'country_code': 'PT',
            'name': 'Azores'
        }
    }
}


COUNTRIES_WITHOUT_POSTAL_CODES = {
    'AE': True,
    'AG': True,
    'AN': True,
    'AO': True,
    'AW': True,
    'BF': True,
    'BI': True,
    'BJ': True,
    'BS': True,
    'BW': True,
    'BZ': True,
    'CD': True,
    'CF': True,
    'CG': True,
    'CI': True,
    'CK': True,
    'CM': True,
    'DJ': True,
    'DM': True,
    'ER': True,
    'FJ': True,
    'GD': True,
    'GH': True,
    'GM': True,
    'GN': True,
    'GQ': True,
    'GY': True,
    'HK': True,
    'IE': True,
    'JM': True,
    'KE': True,
    'KI': True,
    'KM': True,
    'KN': True,
    'KP': True,
    'LC': True,
    'ML': True,
    'MO': True,
    'MR': True,
    'MS': True,
    'MU': True,
    'MW': True,
    'NR': True,
    'NU': True,
    'PA': True,
    'QA': True,
    'RW': True,
    'SA': True,
    'SB': True,
    'SC': True,
    'SL': True,
    'SO': True,
    'SR': True,
    'ST': True,
    'SY': True,
    'TF': True,
    'TK': True,
    'TL': True,
    'TO': True,
    'TT': True,
    'TV': True,
    'TZ': True,
    'UG': True,
    'VU': True,
    'YE': True,
    'ZA': True,
    'ZW': True
}



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import re
from decimal import Decimal

try:
    # Python 2
    str_cls = unicode
except (NameError):
    # Python 3
    str_cls = str

from . import rates
from .errors import UndefinitiveError


def calculate_rate(phone_number, address_country_code=None, address_exception=None):
    """
    Calculates the VAT rate based on a telephone number

    :param phone_number:
        The string phone number, in international format with leading +

    :param address_country_code:
        The user's country_code, as detected from billing_address or
        declared_residence. This prevents an UndefinitiveError from being
        raised.

    :param address_exception:
        The user's exception name, as detected from billing_address or
        declared_residence. This prevents an UndefinitiveError from being
        raised.

    :raises:
        ValueError - error with phone number provided
        UndefinitiveError - when no address_country_code and address_exception are provided and the phone number area code matching isn't specific enough

    :return:
        A tuple of (Decimal percentage rate, country code, exception name [or None])
    """

    if not phone_number:
        raise ValueError('No phone number provided')

    if not isinstance(phone_number, str_cls):
        raise ValueError('Phone number is not a string')

    phone_number = phone_number.strip()
    phone_number = re.sub('[^+0-9]', '', phone_number)

    if not phone_number or phone_number[0] != '+':
        raise ValueError('Phone number is not in international format with a leading +')

    phone_number = phone_number[1:]

    if not phone_number:
        raise ValueError('Phone number does not appear to contain any digits')

    country_code = _lookup_country_code(phone_number)
    if not country_code:
        raise ValueError('Phone number does not appear to be a valid international phone number')

    if country_code in CALLING_CODE_EXCEPTIONS:
        for info in CALLING_CODE_EXCEPTIONS[country_code]:
            if not re.match(info['regex'], phone_number):
                continue

            mapped_country = info['country_code']
            mapped_name = info['name']

            if not info['definitive']:
                if address_country_code is None:
                    raise UndefinitiveError('It is not possible to determine the users VAT rates based on the information provided')

                if address_country_code != mapped_country:
                    continue

                if address_exception != info['name']:
                    continue

            rate = rates.BY_COUNTRY[mapped_country]['exceptions'][mapped_name]
            return (rate, mapped_country, mapped_name)

    if country_code not in rates.BY_COUNTRY:
        return (Decimal('0.0'), country_code, None)

    return (rates.BY_COUNTRY[country_code]['rate'], country_code, None)


def _lookup_country_code(phone_number):
    """
    Accepts an international form of a phone number (+ followed by digits),
    and returns a two-character country code.

    :param phone_number:
        The string phone number, in international format with leading +

    :return:
        A two-character string or None if no match
    """

    leading_digit = phone_number[0]

    if leading_digit not in CALLING_CODE_MAPPING:
        return None

    for mapping in CALLING_CODE_MAPPING[leading_digit]:
        if not re.match(mapping['regex'], phone_number):
            continue
        return mapping['country_code']

    return None


# A list of regular expressions to map against an internation phone number that
# has had the leading + stripped off.
#
# The mapping is in the form:
#
# {
#     digit: [
#         {
#             'regex': regex,
#             'country_code': two character country code
#         }
#     ]
# }
#
# The values are a list so that more specific regexes will be matched first.
# This is necessary since sometimes multiple countries use the same
# international calling code prefix.
CALLING_CODE_MAPPING = {
    '1': [
        {
            'regex': '1(204|226|236|249|250|289|306|343|365|387|403|416|418|431|437|438|450|506|514|519|548|579|581|587|600|604|613|622|633|639|644|647|655|672|677|688|705|709|742|778|780|782|807|819|825|867|873|902|905)',
            'country_code': 'CA'
        },
        {
            'regex': '1268',
            'country_code': 'AG'
        },
        {
            'regex': '1264',
            'country_code': 'AI'
        },
        {
            'regex': '1684',
            'country_code': 'AS'
        },
        {
            'regex': '1246',
            'country_code': 'BB'
        },
        {
            'regex': '1441',
            'country_code': 'BM'
        },
        {
            'regex': '1242',
            'country_code': 'BS'
        },
        {
            'regex': '1767',
            'country_code': 'DM'
        },
        {
            'regex': '1(809|829|849)',
            'country_code': 'DO'
        },
        {
            'regex': '1473',
            'country_code': 'GD'
        },
        {
            'regex': '1671',
            'country_code': 'GU'
        },
        {
            'regex': '1876',
            'country_code': 'JM'
        },
        {
            'regex': '1869',
            'country_code': 'KN'
        },
        {
            'regex': '1345',
            'country_code': 'KY'
        },
        {
            'regex': '1758',
            'country_code': 'LC'
        },
        {
            'regex': '1670',
            'country_code': 'MP'
        },
        {
            'regex': '1664',
            'country_code': 'MS'
        },
        {
            'regex': '1(939|787)',
            'country_code': 'PR'
        },
        {
            'regex': '1721',
            'country_code': 'SX'
        },
        {
            'regex': '1649',
            'country_code': 'TC'
        },
        {
            'regex': '1868',
            'country_code': 'TT'
        },
        {
            'regex': '1784',
            'country_code': 'VC'
        },
        {
            'regex': '1284',
            'country_code': 'VG'
        },
        {
            'regex': '1340',
            'country_code': 'VI'
        },
        {
            'regex': '1',
            'country_code': 'US'
        }
    ],
    '2': [
        {
            'regex': '20',
            'country_code': 'EG'
        },
        {
            'regex': '211',
            'country_code': 'SS'
        },
        {
            'regex': '212(5288|5289)',
            'country_code': 'EH'
        },
        {
            'regex': '212',
            'country_code': 'MA'
        },
        {
            'regex': '213',
            'country_code': 'DZ'
        },
        {
            'regex': '216',
            'country_code': 'TN'
        },
        {
            'regex': '218',
            'country_code': 'LY'
        },
        {
            'regex': '220',
            'country_code': 'GM'
        },
        {
            'regex': '221',
            'country_code': 'SN'
        },
        {
            'regex': '222',
            'country_code': 'MR'
        },
        {
            'regex': '223',
            'country_code': 'ML'
        },
        {
            'regex': '224',
            'country_code': 'GN'
        },
        {
            'regex': '225',
            'country_code': 'CI'
        },
        {
            'regex': '226',
            'country_code': 'BF'
        },
        {
            'regex': '227',
            'country_code': 'NE'
        },
        {
            'regex': '228',
            'country_code': 'TG'
        },
        {
            'regex': '229',
            'country_code': 'BJ'
        },
        {
            'regex': '230',
            'country_code': 'MU'
        },
        {
            'regex': '231',
            'country_code': 'LR'
        },
        {
            'regex': '232',
            'country_code': 'SL'
        },
        {
            'regex': '233',
            'country_code': 'GH'
        },
        {
            'regex': '234',
            'country_code': 'NG'
        },
        {
            'regex': '235',
            'country_code': 'TD'
        },
        {
            'regex': '236',
            'country_code': 'CF'
        },
        {
            'regex': '237',
            'country_code': 'CM'
        },
        {
            'regex': '238',
            'country_code': 'CV'
        },
        {
            'regex': '239',
            'country_code': 'ST'
        },
        {
            'regex': '240',
            'country_code': 'GQ'
        },
        {
            'regex': '241',
            'country_code': 'GA'
        },
        {
            'regex': '242',
            'country_code': 'CG'
        },
        {
            'regex': '243',
            'country_code': 'CD'
        },
        {
            'regex': '244',
            'country_code': 'AO'
        },
        {
            'regex': '245',
            'country_code': 'GW'
        },
        {
            'regex': '246',
            'country_code': 'IO'
        },
        {
            'regex': '247',
            'country_code': 'AC'
        },
        {
            'regex': '248',
            'country_code': 'SC'
        },
        {
            'regex': '249',
            'country_code': 'SD'
        },
        {
            'regex': '250',
            'country_code': 'RW'
        },
        {
            'regex': '251',
            'country_code': 'ET'
        },
        {
            'regex': '252',
            'country_code': 'SO'
        },
        {
            'regex': '253',
            'country_code': 'DJ'
        },
        {
            'regex': '254',
            'country_code': 'KE'
        },
        {
            'regex': '255',
            'country_code': 'TZ'
        },
        {
            'regex': '256',
            'country_code': 'UG'
        },
        {
            'regex': '257',
            'country_code': 'BI'
        },
        {
            'regex': '258',
            'country_code': 'MZ'
        },
        {
            'regex': '260',
            'country_code': 'ZM'
        },
        {
            'regex': '261',
            'country_code': 'MG'
        },
        {
            'regex': '262269',
            'country_code': 'YT'
        },
        {
            'regex': '262',
            'country_code': 'RE'
        },
        {
            'regex': '263',
            'country_code': 'ZW'
        },
        {
            'regex': '264',
            'country_code': 'NA'
        },
        {
            'regex': '265',
            'country_code': 'MW'
        },
        {
            'regex': '266',
            'country_code': 'LS'
        },
        {
            'regex': '267',
            'country_code': 'BW'
        },
        {
            'regex': '268',
            'country_code': 'SZ'
        },
        {
            'regex': '269',
            'country_code': 'KM'
        },
        {
            'regex': '27',
            'country_code': 'ZA'
        },
        {
            'regex': '290',
            'country_code': 'SH'
        },
        {
            'regex': '291',
            'country_code': 'ER'
        },
        {
            'regex': '297',
            'country_code': 'AW'
        },
        {
            'regex': '298',
            'country_code': 'FO'
        },
        {
            'regex': '299',
            'country_code': 'GL'
        }
    ],
    '3': [
        {
            'regex': '30',
            'country_code': 'GR'
        },
        {
            'regex': '31',
            'country_code': 'NL'
        },
        {
            'regex': '32',
            'country_code': 'BE'
        },
        {
            'regex': '33',
            'country_code': 'FR'
        },
        {
            'regex': '34',
            'country_code': 'ES'
        },
        {
            'regex': '350',
            'country_code': 'GI'
        },
        {
            'regex': '351',
            'country_code': 'PT'
        },
        {
            'regex': '352',
            'country_code': 'LU'
        },
        {
            'regex': '353',
            'country_code': 'IE'
        },
        {
            'regex': '354',
            'country_code': 'IS'
        },
        {
            'regex': '355',
            'country_code': 'AL'
        },
        {
            'regex': '356',
            'country_code': 'MT'
        },
        {
            'regex': '357',
            'country_code': 'CY'
        },
        {  # Åland Islands (to exclude from FI)
            'regex': '35818',
            'country_code': 'AX'
        },
        {
            'regex': '358',
            'country_code': 'FI'
        },
        {
            'regex': '359',
            'country_code': 'BG'
        },
        {
            'regex': '36',
            'country_code': 'HU'
        },
        {
            'regex': '370',
            'country_code': 'LT'
        },
        {
            'regex': '371',
            'country_code': 'LV'
        },
        {
            'regex': '372',
            'country_code': 'EE'
        },
        {
            'regex': '373',
            'country_code': 'MD'
        },
        {
            'regex': '374',
            'country_code': 'AM'
        },
        {
            'regex': '375',
            'country_code': 'BY'
        },
        {
            'regex': '376',
            'country_code': 'AD'
        },
        {
            'regex': '377(44|45)',
            'country_code': 'XK'
        },
        {
            'regex': '377',
            'country_code': 'MC'
        },
        {
            'regex': '378',
            'country_code': 'SM'
        },
        {
            'regex': '379',
            'country_code': 'VA'
        },
        {
            'regex': '380',
            'country_code': 'UA'
        },
        {
            'regex': '381(28|29|38|39)',
            'country_code': 'XK'
        },
        {
            'regex': '381',
            'country_code': 'RS'
        },
        {
            'regex': '382',
            'country_code': 'ME'
        },
        {
            'regex': '383',
            'country_code': 'XK'
        },
        {
            'regex': '385',
            'country_code': 'HR'
        },
        {
            'regex': '386(43|49)',
            'country_code': 'XK'
        },
        {
            'regex': '386',
            'country_code': 'SI'
        },
        {
            'regex': '387',
            'country_code': 'BA'
        },
        {
            'regex': '389',
            'country_code': 'MK'
        },
        {
            'regex': '3906698',
            'country_code': 'VA'
        },
        {
            'regex': '39',
            'country_code': 'IT'
        }
    ],
    '4': [
        {
            'regex': '40',
            'country_code': 'RO'
        },
        {
            'regex': '41',
            'country_code': 'CH'
        },
        {
            'regex': '420',
            'country_code': 'CZ'
        },
        {
            'regex': '421',
            'country_code': 'SK'
        },
        {
            'regex': '423',
            'country_code': 'LI'
        },
        {
            'regex': '43',
            'country_code': 'AT'
        },
        {  # Guernsey (to exclude from GB)
            'regex': '44(148|7781|7839|7911)',
            'country_code': 'GG'
        },
        {  # Jersey (to exclude from GB)
            'regex': '44(153|7509|7797|7937|7700|7829)',
            'country_code': 'JE'
        },
        {  # Isle of Man
            'regex': '44(162|7624|7524|7924)',
            'country_code': 'IM'
        },
        {
            'regex': '44',
            'country_code': 'GB'
        },
        {
            'regex': '45',
            'country_code': 'DK'
        },
        {
            'regex': '46',
            'country_code': 'SE'
        },
        {
            'regex': '47',
            'country_code': 'NO'
        },
        {
            'regex': '48',
            'country_code': 'PL'
        },
        {
            'regex': '49',
            'country_code': 'DE'
        }
    ],
    '5': [
        {
            'regex': '500',
            'country_code': 'FK'
        },
        {
            'regex': '501',
            'country_code': 'BZ'
        },
        {
            'regex': '502',
            'country_code': 'GT'
        },
        {
            'regex': '503',
            'country_code': 'SV'
        },
        {
            'regex': '504',
            'country_code': 'HN'
        },
        {
            'regex': '505',
            'country_code': 'NI'
        },
        {
            'regex': '506',
            'country_code': 'CR'
        },
        {
            'regex': '507',
            'country_code': 'PA'
        },
        {
            'regex': '508',
            'country_code': 'PM'
        },
        {
            'regex': '509',
            'country_code': 'HT'
        },
        {
            'regex': '51',
            'country_code': 'PE'
        },
        {
            'regex': '52',
            'country_code': 'MX'
        },
        {
            'regex': '53',
            'country_code': 'CU'
        },
        {
            'regex': '54',
            'country_code': 'AR'
        },
        {
            'regex': '55',
            'country_code': 'BR'
        },
        {
            'regex': '56',
            'country_code': 'CL'
        },
        {
            'regex': '57',
            'country_code': 'CO'
        },
        {
            'regex': '58',
            'country_code': 'VE'
        },
        {
            'regex': '590(590(51|52|58|77|87)|690(10|22|27|66|77|87|88))',
            'country_code': 'MF'
        },
        {
            'regex': '590590(27|29)',
            'country_code': 'BL'
        },
        {
            'regex': '590',
            'country_code': 'GP'
        },
        {
            'regex': '591',
            'country_code': 'BO'
        },
        {
            'regex': '592',
            'country_code': 'GY'
        },
        {
            'regex': '593',
            'country_code': 'EC'
        },
        {
            'regex': '594',
            'country_code': 'GF'
        },
        {
            'regex': '595',
            'country_code': 'PY'
        },
        {
            'regex': '596',
            'country_code': 'MQ'
        },
        {
            'regex': '597',
            'country_code': 'SR'
        },
        {
            'regex': '598',
            'country_code': 'UY'
        },
        {
            'regex': '5999',
            'country_code': 'CW'
        },
        {
            'regex': '599',
            'country_code': 'BQ'
        }
    ],
    '6': [
        {
            'regex': '60',
            'country_code': 'MY'
        },
        {
            'regex': '6189164',
            'country_code': 'CX'
        },
        {
            'regex': '6189162',
            'country_code': 'CC'
        },
        {
            'regex': '61',
            'country_code': 'AU'
        },
        {
            'regex': '62',
            'country_code': 'ID'
        },
        {
            'regex': '63',
            'country_code': 'PH'
        },
        {
            'regex': '64',
            'country_code': 'NZ'
        },
        {
            'regex': '65',
            'country_code': 'SG'
        },
        {
            'regex': '66',
            'country_code': 'TH'
        },
        {
            'regex': '670',
            'country_code': 'TL'
        },
        {
            'regex': '6723',
            'country_code': 'NF'
        },
        {
            'regex': '6721',
            'country_code': 'AQ'
        },
        {
            'regex': '673',
            'country_code': 'BN'
        },
        {
            'regex': '674',
            'country_code': 'NR'
        },
        {
            'regex': '675',
            'country_code': 'PG'
        },
        {
            'regex': '676',
            'country_code': 'TO'
        },
        {
            'regex': '677',
            'country_code': 'SB'
        },
        {
            'regex': '678',
            'country_code': 'VU'
        },
        {
            'regex': '679',
            'country_code': 'FJ'
        },
        {
            'regex': '680',
            'country_code': 'PW'
        },
        {
            'regex': '681',
            'country_code': 'WF'
        },
        {
            'regex': '682',
            'country_code': 'CK'
        },
        {
            'regex': '683',
            'country_code': 'NU'
        },
        {
            'regex': '685',
            'country_code': 'WS'
        },
        {
            'regex': '686',
            'country_code': 'KI'
        },
        {
            'regex': '687',
            'country_code': 'NC'
        },
        {
            'regex': '688',
            'country_code': 'TV'
        },
        {
            'regex': '689',
            'country_code': 'PF'
        },
        {
            'regex': '690',
            'country_code': 'TK'
        },
        {
            'regex': '691',
            'country_code': 'FM'
        },
        {
            'regex': '692',
            'country_code': 'MH'
        }
    ],
    '7': [
        {
            'regex': '7(840|940)',
            'country_code': 'GE'
        },
        {
            'regex': '7[3489]',
            'country_code': 'RU'
        },
        {
            'regex': '7[67]',
            'country_code': 'KZ'
        }
    ],
    '8': [
        {
            'regex': '81',
            'country_code': 'JP'
        },
        {
            'regex': '82',
            'country_code': 'KR'
        },
        {
            'regex': '84',
            'country_code': 'VN'
        },
        {
            'regex': '850',
            'country_code': 'KP'
        },
        {
            'regex': '852',
            'country_code': 'HK'
        },
        {
            'regex': '853',
            'country_code': 'MO'
        },
        {
            'regex': '855',
            'country_code': 'KH'
        },
        {
            'regex': '856',
            'country_code': 'LA'
        },
        {
            'regex': '86',
            'country_code': 'CN'
        },
        {
            'regex': '880',
            'country_code': 'BD'
        },
        {
            'regex': '886',
            'country_code': 'TW'
        }
    ],
    '9': [
        {
            'regex': '90',
            'country_code': 'TR'
        },
        {
            'regex': '91',
            'country_code': 'IN'
        },
        {
            'regex': '92',
            'country_code': 'PK'
        },
        {
            'regex': '93',
            'country_code': 'AF'
        },
        {
            'regex': '94',
            'country_code': 'LK'
        },
        {
            'regex': '95',
            'country_code': 'MM'
        },
        {
            'regex': '960',
            'country_code': 'MV'
        },
        {
            'regex': '961',
            'country_code': 'LB'
        },
        {
            'regex': '962',
            'country_code': 'JO'
        },
        {
            'regex': '963',
            'country_code': 'SY'
        },
        {
            'regex': '964',
            'country_code': 'IQ'
        },
        {
            'regex': '965',
            'country_code': 'KW'
        },
        {
            'regex': '966',
            'country_code': 'SA'
        },
        {
            'regex': '967',
            'country_code': 'YE'
        },
        {
            'regex': '968',
            'country_code': 'OM'
        },
        {
            'regex': '970',
            'country_code': 'PS'
        },
        {
            'regex': '971',
            'country_code': 'AE'
        },
        {
            'regex': '972',
            'country_code': 'IL'
        },
        {
            'regex': '973',
            'country_code': 'BH'
        },
        {
            'regex': '974',
            'country_code': 'QA'
        },
        {
            'regex': '975',
            'country_code': 'BT'
        },
        {
            'regex': '976',
            'country_code': 'MN'
        },
        {
            'regex': '977',
            'country_code': 'NP'
        },
        {
            'regex': '98',
            'country_code': 'IR'
        },
        {
            'regex': '992',
            'country_code': 'TJ'
        },
        {
            'regex': '993',
            'country_code': 'TM'
        },
        {
            'regex': '994',
            'country_code': 'AZ'
        },
        {
            'regex': '995',
            'country_code': 'GE'
        },
        {
            'regex': '996',
            'country_code': 'KG'
        },
        {
            'regex': '998',
            'country_code': 'UZ'
        }
    ]
}


# The country_code key is included with these exceptions since some cities have
# phone service from more than one country.
#
# The main dict key is the country code, as matched from CALLING_CODE_MAPPING
CALLING_CODE_EXCEPTIONS = {
    'AT': [
        {
            'regex': '435676',
            'country_code': 'AT',
            'name': 'Jungholz',
            'definitive': True
        },
        {
            'regex': '435517',
            'country_code': 'AT',
            'name': 'Mittelberg',
            'definitive': False
        }
    ],
    'CH': [
        {
            'regex': '4152',
            'country_code': 'DE',
            'name': 'Büsingen am Hochrhein',
            'definitive': False
        },
        {
            'regex': '4191',
            'country_code': 'IT',
            'name': "Campione d'Italia",
            'definitive': False
        }
    ],
    'DE': [
        {
            'regex': '494725',
            'country_code': 'DE',
            'name': 'Heligoland',
            'definitive': True
        },
        {
            'regex': '497734',
            'country_code': 'DE',
            'name': 'Büsingen am Hochrhein',
            'definitive': False
        }
    ],
    'ES': [
        {
            'regex': '34(822|828|922|928)',
            'country_code': 'ES',
            'name': 'Canary Islands',
            'definitive': True
        },
        {
            'regex': '34956',
            'country_code': 'ES',
            'name': 'Ceuta',
            'definitive': False
        },
        {
            'regex': '34952',
            'country_code': 'ES',
            'name': 'Melilla',
            'definitive': False
        }
    ],
    'GR': [
        {
            # http://www.mountathosinfos.gr/pages/agionoros/telefonbook.en.html
            # http://www.athosfriends.org/PilgrimsGuide/information/#telephones
            'regex': '3023770(23|41488|41462|22586|24039|94098)',
            'country_code': 'GR',
            'name': 'Mount Athos',
            'definitive': True
        }
    ],
    'IT': [
        {
            'regex': '390342',
            'country_code': 'IT',
            'name': 'Livigno',
            'definitive': False
        }
    ],
    'PT': [
        {
            'regex': '35129[256]',
            'country_code': 'PT',
            'name': 'Azores',
            'definitive': True
        },
        {
            'regex': '351291',
            'country_code': 'PT',
            'name': 'Madeira',
            'definitive': True
        }
    ]
}



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from decimal import Decimal


# The rates used here are pull from the following sources December 17, 2014:
#
# http://ec.europa.eu/taxation_customs/resources/documents/taxation/vat/how_vat_works/rates/vat_rates_en.pdf
# http://www.skatteetaten.no/en/Bedrift-og-organisasjon/Merverdiavgift/Guide-to-Value-Added-Tax-in-Norway/?chapter=3732#kapitteltekst
# http://en.wikipedia.org/wiki/Special_member_state_territories_and_the_European_Union
#
# The following is an extrapolation of special EU VAT rates and exemptions
# listed in the above PDF.
#
#   VAT Does Not Apply
#
#     Countries
#
#       FO - Faroe Islands - Denmark
#       GL - Greenland - Denmark
#       AX - Åland Islands - Finland
#       GF - French Guiana - France
#
#     Cities or regions
#
#       Canary Islands - Spain
#       Melilla - Spain
#       Ceuta - Spain
#       Büsingen am Hochrhein - Germany
#       Heligoland - Germany
#       Mount Athos - Greece
#       Campione d'Italia - Italy
#       Livigno - Italy
#
#     French overseas departments - these have a VAT similar to EU, but they are
#     not actually part of the EU VAT system, so you do not have to collect VAT
#     for e-services. The local VAT is 8.5%.
#     http://ec.europa.eu/taxation_customs/taxation/other_taxes/dock_dues/index_en.htm
#
#       GP - Guadeloupe
#       RE - Réunion
#       MQ - Martinique
#
#   Special VAT Rates
#
#     Monaco - France - 20%
#     Isle of Man - United Kingdom - 20%
#     Azores - Portugal - 18%
#     Madeira - Portugal - 22%
#     Akrotiri - Cyprus - 19%
#     Dhekelia - Cyprus - 19%
#     Jungholz - Austria - 19%
#     Mittelberg - Austria - 19%


# There are country entries and exceptions entries for places that are listed
# on the VAT exceptions list. A value of None means no VAT is to be collected.
BY_COUNTRY = {
    'AT': {  # Austria
        'rate': Decimal('0.20'),
        'exceptions': {
            'Jungholz': Decimal('0.19'),
            'Mittelberg': Decimal('0.19')
        }
    },
    'BE': {  # Belgium
        'rate': Decimal('0.21')
    },
    'BG': {  # Bulgaria
        'rate': Decimal('0.20')
    },
    'CY': {  # Cyprus
        'rate': Decimal('0.19')
    },
    'CZ': {  # Czech Republic
        'rate': Decimal('0.21')
    },
    'DE': {  # Germany
        'rate': Decimal('0.19'),
        'exceptions': {
            'Büsingen am Hochrhein': Decimal('0.0'),
            'Heligoland': Decimal('0.0')
        }
    },
    'DK': {  # Denmark
        'rate': Decimal('0.25')
    },
    'EE': {  # Estonia
        'rate': Decimal('0.20')
    },
    'ES': {  # Spain
        'rate': Decimal('0.21'),
        'exceptions': {
            'Canary Islands': Decimal('0.0'),
            'Ceuta': Decimal('0.0'),
            'Melilla': Decimal('0.0')
        }
    },
    'FI': {  # Finland
        'rate': Decimal('0.24')
    },
    'FR': {  # France
        'rate': Decimal('0.20')
    },
    'GB': {  # United Kingdom
        'rate': Decimal('0.20'),
        'exceptions': {
            # UK RAF Bases in Cyprus are taxed at Cyprus rate
            'Akrotiri': (Decimal('0.19'), 'CY', None),
            'Dhekelia': (Decimal('0.19'), 'CY', None)
        }
    },
    'GR': {  # Greece
        'rate': Decimal('0.23'),
        'exceptions': {
            'Mount Athos': Decimal('0.0')
        }
    },
    'HR': {  # Croatia
        'rate': Decimal('0.25')
    },
    'HU': {  # Hungary
        'rate': Decimal('0.27')
    },
    'IE': {  # Ireland
        'rate': Decimal('0.23')
    },
    'IT': {  # Italy
        'rate': Decimal('0.22'),
        'exceptions': {
            "Campione d'Italia": Decimal('0.0'),
            'Livigno': Decimal('0.0')
        }
    },
    'LT': {  # Lithuania
        'rate': Decimal('0.21')
    },
    'LU': {  # Luxembourg
        'rate': Decimal('0.15')
    },
    'LV': {  # Latvia
        'rate': Decimal('0.21')
    },
    'MT': {  # Malta
        'rate': Decimal('0.18')
    },
    'NL': {  # Netherlands
        'rate': Decimal('0.21')
    },
    'PL': {  # Poland
        'rate': Decimal('0.23')
    },
    'PT': {  # Portugal
        'rate': Decimal('0.23'),
        'exceptions': {
            'Azores': Decimal('0.0'),
            'Madeira': Decimal('0.0')
        }
    },
    'RO': {  # Romania
        'rate': Decimal('0.24')
    },
    'SE': {  # Sweden
        'rate': Decimal('0.25')
    },
    'SI': {  # Slovenia
        'rate': Decimal('0.22')
    },
    'SK': {  # Slovakia
        'rate': Decimal('0.20')
    },

    # Countries associated with EU countries that have a special VAT rate
    'MC': {  # Monaco - France
        'rate': Decimal('0.20')
    },
    'IM': {  # Isle of Man - United Kingdom
        'rate': Decimal('0.20')
    },

    # Non-EU with their own VAT collection requirements
    'NO': {  # Norway
        'rate': Decimal('0.25')
    }
}



from setuptools import setup, find_packages
import vat_moss


setup(
    name='vat_moss',
    version=vat_moss.__version__,

    description='Tools for VAT MOSS and Norway VAT on digital services.',
    long_description='Docs for this project are maintained at https://github.com/wbond/vat_moss-python#readme.',

    url='https://github.com/wbond/vat_moss-python',

    author='wbond',
    author_email='will@wbond.net',

    license='MIT',

    classifiers=[
        'Development Status :: 4 - Beta',

        'Intended Audience :: Developers',

        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],

    keywords='vat',

    packages=find_packages(exclude=['tests*'])
)
